package com.dsic.gj_erp.controller.kban;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.resource.ResourceUtil;
import cn.hutool.core.util.StrUtil;
import com.dsic.gj_erp.bean.jcsj.DmCbxxp;
import com.dsic.gj_erp.service.jcsj.DmBomService;
import com.dsic.gj_erp.service.jcsj.DmCbxxpService;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dsic.gj_erp.bean.ResultBean;
import com.dsic.gj_erp.bean.jcsj.DmBzry;
import com.dsic.gj_erp.bean.jhgk.DmYdjh;
import com.dsic.gj_erp.bean.jhgk.DmYdjhxf;
import com.dsic.gj_erp.exception.ServiceException;
import com.dsic.gj_erp.service.KbanService;
import com.dsic.gj_erp.service.jcsj.DmBzryService;
import com.dsic.gj_erp.service.jhgk.DmSygdMxService;
import com.dsic.gj_erp.service.sy.impl.SyMsgService;
import com.dsic.gj_erp.service.zyjh.ZyjhService;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjUtil;
import lombok.AllArgsConstructor;

/**
 * 各作业区划分规则
 * 一区:上料
 * 二区:一垮,三跨 切割
 * 三区:二跨
 * 四区:四五六垮
 * 五区:三跨 曲加工
 * 六区:光电,吊车
 * 七垮:型材
 */
@RestController
@RequestMapping("kban")
@AllArgsConstructor
public class KbanController {

    private final ZyjhService zyjhService;
    private final DmBzryService cqService;
    private final KbanService kbanService;
    private final SyMsgService msgService;
    private final DmSygdMxService service;


    /**
     * 船只批量获取接口
     */
    @GetMapping("/getProject")
    public ResultBean<?> getProject() {
        return new ResultBean<>(kbanService.getProjects());
    }

    /**
     * 根据船号和批量获取pdf列表
     */
    @GetMapping("/pdfList")
    public ResultBean<?> pdfList(String dcCh,String dcPl) {
        String absolutePath = FileUtil.getWebRoot().getAbsolutePath();
        List<File> ls;
        try {
            ls = FileUtil.loopFiles(StrUtil.format("{}/../static/upload/{}/{}/PDF",absolutePath, dcCh, dcPl));
        }catch (Exception e){
            ls = FileUtil.loopFiles(StrUtil.format("{}/static/upload/{}/{}/PDF",absolutePath, dcCh, dcPl));
        }

        return new ResultBean<>(ls.stream().map(FileUtil::getPrefix));
    }

    /**
     * 30天作业区完成情况折线图
     */
    @GetMapping("/today_zyjhzx")
    public ResultBean<?> todayXZyjhzx() {
        return new ResultBean<>(this.kbanService.todayZyjhzx());
    }

    /**
     * 30天作业区完成情况折线图
     */
    @GetMapping("/zyjhzx")
    public ResultBean<?> zyjhzx() {
        return new ResultBean<>(this.kbanService.zyjhzx());
    }

    /**
     * 前方需求汇总表
     */
    @GetMapping("/zyjhzxTable")
    public ResultBean<?> zyjhzxTable() {
        Map<String, Object> map=this.kbanService.zyjhzxTable();
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        List<DmYdjh> list = zyjhService
                .list(Wrappers.<DmYdjh>lambdaQuery().ge(DmYdjh::getZt, 2).between(DmYdjh::getJssj, from, to));

        AtomicInteger 上料计划 = new AtomicInteger(0);
        AtomicInteger 上料完成 = new AtomicInteger(0);
        HashMap<String, Object> 上料 = new HashMap<>();

        AtomicInteger 二区计划1k = new AtomicInteger(0);
        AtomicInteger 二区完成1k = new AtomicInteger(0);
        AtomicInteger 二区计划3k = new AtomicInteger(0);
        AtomicInteger 二区完成3k = new AtomicInteger(0);
        HashMap<String, Object> 二区1k = new HashMap<>();
        HashMap<String, Object> 二区3k = new HashMap<>();

        AtomicInteger 三区计划 = new AtomicInteger(0);
        AtomicInteger 三区完成 = new AtomicInteger(0);
        HashMap<String, Object> 三区 = new HashMap<>();

        AtomicInteger 四区计划 = new AtomicInteger(0);
        AtomicInteger 四区完成 = new AtomicInteger(0);
        HashMap<String, Object> 四区 = new HashMap<>();

        list.forEach(item -> {
            String subStr = item.getTzbh().substring(5, 6);
            int zt = 0;

            try {
                zt = Integer.parseInt(item.getZt());
            } catch (Exception e) {
                e.printStackTrace();
            }

            if (zt == 11) {
                上料计划.incrementAndGet();
            }

            if (zt > 11) {
                上料完成.incrementAndGet();
            }

            // 二区
            if ("1".equals(subStr)) {
                if (zt >= 2) {
                    二区计划1k.incrementAndGet();
                }
                if (zt >= 62) {
                    二区完成1k.incrementAndGet();
                }
            }

            if ("3".equals(subStr)) {
                if (zt >= 2) {
                    二区计划3k.incrementAndGet();
                }
                if (zt >= 62) {
                    二区完成3k.incrementAndGet();
                }
            }

            // 三区
            if ("2".equals(subStr)) {
                if (zt >= 2) {
                    三区计划.incrementAndGet();
                }
                if (zt >= 62) {
                    三区完成.incrementAndGet();
                }
            }

            // 四区
            if ("4".equals(subStr)) {
                if (zt >= 2) {
                    四区计划.incrementAndGet();
                }
                if (zt >= 62) {
                    四区完成.incrementAndGet();
                }
            }
        });

        ArrayList<Object> result = new ArrayList<>();
        // 上料.put("zyq", "一区");
        // 上料.put("sl", 上料计划.get());
        // 上料.put("wc", 上料完成.get());
        // result.add(上料);

        二区1k.put("zyq", "二区");
        二区1k.put("kw", "一垮");
        二区1k.put("sl", 二区计划1k.get());
        二区1k.put("wc", 二区完成1k.get());
        result.add(二区1k);

        二区3k.put("zyq", "二区");
        二区3k.put("kw", "三垮");
        二区3k.put("sl", 二区计划3k.get());
        二区3k.put("wc", 二区完成3k.get());
        result.add(二区3k);

        三区.put("zyq", "三区");
        三区.put("kw", "二垮");
        三区.put("sl", 三区计划.get());
        三区.put("wc", 三区完成.get());
        result.add(三区);

        四区.put("zyq", "四区");
        四区.put("kw", "四五六垮");
        四区.put("sl", 四区计划.get());
        四区.put("wc", 四区完成.get());
        result.add(四区);

        map.put("detail", result);
        return new ResultBean<>(map);
    }

    /**
     * 前方需求柱状图
     */
    @GetMapping("/qfxqWc")
    public ResultBean<?> qfxqWcqk() {
        return new ResultBean<>(this.kbanService.qfxqWcqk());
    }

    /**
     * 前方需求汇总表
     */
    @GetMapping("/qfxqTable")
    public ResultBean<?> qfxqTable() {
        return new ResultBean<>(this.kbanService.qfxqTable());
    }

    /**
     * 作业区完成情况饼图
     */
    @GetMapping("/zxqwcqk")
    public ResultBean<?> getZxqwcqk(){
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        List<DmYdjh> list=zyjhService.list(Wrappers.<DmYdjh>lambdaQuery().ge(DmYdjh::getZt,2).between(DmYdjh::getJssj,from,to));

        AtomicInteger 上料计划 = new AtomicInteger(0);
        AtomicInteger 上料完成 = new AtomicInteger(0);
        HashMap<String, Object> 上料 = new HashMap<>();

        AtomicInteger 二区计划 = new AtomicInteger(0);
        AtomicInteger 二区完成 = new AtomicInteger(0);
        HashMap<String, Object> 二区 = new HashMap<>();

        AtomicInteger 三区计划 = new AtomicInteger(0);
        AtomicInteger 三区完成 = new AtomicInteger(0);
        HashMap<String, Object> 三区 = new HashMap<>();

        AtomicInteger 四区计划 = new AtomicInteger(0);
        AtomicInteger 四区完成 = new AtomicInteger(0);
        HashMap<String, Object> 四区 = new HashMap<>();

        list.forEach(item->{
            String subStr=item.getTzbh().substring(5,6);
            int zt=0;

            try {
                zt = Integer.parseInt(item.getZt());
            }catch (Exception e){
                e.printStackTrace();
            }

            if (zt==11){
                上料计划.incrementAndGet();
            }

            if (zt>11){
                上料完成.incrementAndGet();
            }

            //二区
            if ("1".equals(subStr)||"3".equals(subStr)){
                if (zt>=2){
                    二区计划.incrementAndGet();
                }
                if (zt>=62){
                    二区完成.incrementAndGet();
                }
            }

            //三区
            if ("2".equals(subStr)){
                if (zt>=2){
                    三区计划.incrementAndGet();
                }
                if (zt>=62){
                    三区完成.incrementAndGet();
                }
            }

            //四区
            if ("4".equals(subStr)){
                if (zt>=2){
                    四区计划.incrementAndGet();
                }
                if (zt>=62){
                    四区完成.incrementAndGet();
                }
            }
        });

        ArrayList<Object> result = new ArrayList<>();
        上料.put("zyq","一区");
        上料.put("wcl",上料计划.get()-上料完成.get());
        上料.put("wc",上料完成.get());
        result.add(上料);

        二区.put("zyq","二区");
        二区.put("wcl",二区计划.get()-二区完成.get());
        二区.put("wc",二区完成.get());
        result.add(二区);

        三区.put("zyq","三区");
        三区.put("wcl",三区计划.get()-三区完成.get());
        三区.put("wc",三区完成.get());
        result.add(三区);

        四区.put("zyq","四区");
        四区.put("wcl",四区计划.get()-四区完成.get());
        四区.put("wc",四区完成.get());
        result.add(四区);

        return new ResultBean<>(result);
    }

    /**
     * 作业区执行情况
     */
    @GetMapping("/zxqZxDetail")
    public ResultBean<?> zxqZxDetail(){
        return new ResultBean<>(this.kbanService.zxqZxDetail());
    }

    /**
     * 作业计划
     */
    @GetMapping("/zyjh")
    public ResultBean<?> getZyjhtj(String zyq){
        return new ResultBean<>(this.kbanService.getZyjhtj(zyq));
    }

    @GetMapping("/ydjh")
    public ResultBean<?> getList(){
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        List<DmYdjhxf> list=  this.service.tqSygd1(from,to);
        return new ResultBean<>(list);
    }

    /**
     * 消息
     */
    @GetMapping("msg")
    public ResultBean<?> getMsg(){
        return new ResultBean<>(msgService.list());
    }

    /**
     * 获取所有未处理批次信息
     */
    @GetMapping("wcl")
    public ResultBean<?> getWcl(){
        return new ResultBean<>(zyjhService.getWcl());
    }

    /**
     * 看板线表计划相关数据
     */
    @GetMapping("xbjh")
    public ResultBean<?> kban(String nf){
        if (ObjUtil.isEmpty(nf)){
            throw new ServiceException(10001,"年份不能为空");
        }

        List<Double> xbjh=kbanService.getEveryMonXbzlByNf(nf);
        List<Map<String, Object>> comp = kbanService.getEveryCompZlByNf(nf);

        HashMap<Object, Object> map = new HashMap<>();
        map.put("xbjh", xbjh);
        map.put("ydjh", comp);
        return new ResultBean<>(map);
    }

    /**
     * 计划跟踪
     */
    @GetMapping("getJhgz")
    public ResultBean<?> getJhgz(){
        return new ResultBean<>(kbanService.getJhgz());
    }

    /**
     * 按照设备编号查询作业计划
     */
    @GetMapping("getZyjhDetailBySbCode")
    public ResultBean<?> getZyjhDetailBySbCode(String code){
        if (ObjUtil.isEmpty(code)){
            throw new ServiceException(10001,"设备编号不能为空");
        }
        return new ResultBean<>(kbanService.getZyjhDetailBySbCode(code));
    }

    /**
     * 统计划线、切割设备本人计划及作业成功
     */
    @GetMapping("getSb")
    public ResultBean<?> getSb(){
        List<Map<String, Object>> sb = kbanService.getSb();
        return new ResultBean<>(sb);
    }

    /**
     * 按照人员工号查询当月所有参与的任务
     */
    @GetMapping("getZjjhByUserCode")
    public ResultBean<?> getZjjhByUserCode(String userCode){
        if (ObjUtil.isEmpty(userCode)){
            throw new ServiceException(10001,"用户不能为空");
        }
        return new ResultBean<>(kbanService.getZjjhByUserCode(userCode));
    }

    /**
     * 按照跨位查询作业计划明细
     */
    @GetMapping("getKwDetailByKw")
    public ResultBean<?> getEveryKw(String kw){
        if (ObjUtil.isEmpty(kw)){
            throw new ServiceException(10001,"垮位不能为空");
        }

        QueryWrapper<DmYdjh> wrapper = new QueryWrapper<>();
        if ("1".equals(kw)){
            wrapper.lambda().ge(DmYdjh::getZt, 11);
        }

        if (Arrays.asList("2","3","4").contains(kw)){
            wrapper.eq("SUBSTRING(tzbh,6,1)", kw);
            wrapper.ge("zt", "42");
        }
        return new ResultBean<>(zyjhService.list(wrapper));
    }

    /**
     * 本月各跨位情况
     */
    @GetMapping("getEveryKw")
    public ResultBean<?> getEveryKw(){
        Map<String, Object> map = kbanService.getEveryKw();
        return new ResultBean<>(map);
    }

    /**
     * 在建船只,各船只进度
     */
    @GetMapping("gcjd")
    public ResultBean<?> gcjd(){
        return new ResultBean<>(zyjhService.gcjd());
    }

    /**
     * 各垮计划完成情况
     */
    @GetMapping("gxjhwcqk")
    public ResultBean<?> gxjhwcqk(){
        List<Map<String, Object>> gxjhwcqk = zyjhService.gxjhwcqk();
        return new ResultBean<>(gxjhwcqk);
    }

    /**
     * 各月份产量(吨)
     */
    @GetMapping("gyfcl")
    public ResultBean<?> gyfcl(){
        return new ResultBean<>(zyjhService.gyfcl());
    }

    /**
     * 计划完成情况(吨)
     */
    @GetMapping("jhwcqk")
    public ResultBean<?> jhwcqk(){
        return new ResultBean<>(zyjhService.jhwcqk());
    }

    /**
     * 各垮产量
     */
    @GetMapping("gekuachaliang")
    public ResultBean<?> gekuachaliang(){
        return new ResultBean<>(zyjhService.gekuachaliang());
    }

    /**
     * 考勤情况
     */
    @GetMapping("kq")
    public ResultBean<?> kq(){
        List<DmBzry> ryList=cqService.list(Wrappers.<DmBzry>lambdaQuery()
                .isNotNull(DmBzry::getUserCode).isNotNull(DmBzry::getZt).ne(DmBzry::getZt,""));

        List<DmBzry> collect = ryList.stream().filter(item -> "01".equals(item.getZt())).collect(Collectors.toList());

        Map<String,Object> map=new HashMap<>();
        map.put("total",ryList.size());
        map.put("rows",ryList);
        map.put("bb", collect.size());
        map.put("eb",ryList.size()-collect.size());
        return new ResultBean<>(map);
    }
}
