package com.dsic.gj_erp.service;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

import org.springframework.stereotype.Service;

import com.dsic.gj_erp.bean.jhgk.DmYdjh;
import com.dsic.gj_erp.mapper.KbanMapper;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import lombok.AllArgsConstructor;

@Service
@AllArgsConstructor
public class KbanService {

    private final KbanMapper kbanMapper;

    public List<Map<String, Object>> getProjects() {
        return kbanMapper.getProjects();
    }

    public Map<String, Object> todayZyjhzx() {
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        Map<String,Object> map=kbanMapper.todayZyjhzx(from, to);
        List<Map<String,Object>> detail=kbanMapper.zxqZxDetail(date.toString("yyyy/MM/dd"));
        map.put("detail", detail);
        return map;

    }

    public List<Map<String, Object>> zyjhzx() {
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return kbanMapper.zyjhzx(from, to);

    }

    public Map<String, Object> zyjhzxTable() {
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return kbanMapper.zyjh(from, to);
    }

    public List<Map<String, Object>> qfxqWcqk() {
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return kbanMapper.qfxqWcqk(from,to);
        
    }

    public Map<String, Object> qfxqTable() {
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        String xq=kbanMapper.qfxq(from,to);
        List<Map<String,Object>> list=kbanMapper.qfxqDetail(from,to);

        Map<String,Object> map=new HashMap<>();
        map.put("xq", xq);
        map.put("detail", list);
        return map;
    }

    public List<Map<String,Object>> getZxqwcqk(){
        return kbanMapper.getZxqwcqk();
    }

    public List<Map<String,Object>> zxqZxDetail(){
        String today=DateUtil.date().toString("yyyy/MM/dd");
        return kbanMapper.zxqZxDetail(today);
    }

    public List<Map<String,Object>> getZyjhtj(String zyq){
        DateTime date = DateUtil.date();
        String from= DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to =DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return kbanMapper.getZyjhtj(from,to,zyq);
    }

    public List<Map<String,Object>> getJhgz(){
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return kbanMapper.getJhgz(from,to);
    }

    public List<DmYdjh> getZyjhDetailBySbCode(String code){
        return kbanMapper.getZyjhDetailBySbCode(code);
    }

    public List<Map<String,Object>> getSb(){
        DateTime date = DateUtil.date();
        return kbanMapper.getSb(date.toString("yyyy/MM/dd"));
    }


    public List<DmYdjh> getZjjhByUserCode(String userCode){
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return kbanMapper.getZjjhByUserCode(userCode,from,to);
    }

    /**
     * 各垮任务情况
     * 派工,报工,未处理
     */
    public Map<String,Object> getEveryKw(){
        Map<String, Object> map = new HashMap<>();
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        List<Map<String, String>> everyKw = kbanMapper.getEveryKw(from, to);
        AtomicInteger sl=new AtomicInteger(0);
        AtomicInteger slc=new AtomicInteger(0);
        AtomicInteger kw1=new AtomicInteger(0);
        AtomicInteger kw1c=new AtomicInteger(0);
        AtomicInteger kw2=new AtomicInteger(0);
        AtomicInteger kw2c=new AtomicInteger(0);
        AtomicInteger kw3=new AtomicInteger(0);
        AtomicInteger kw3c=new AtomicInteger(0);
        AtomicInteger kw4=new AtomicInteger(0);
        AtomicInteger kw4c=new AtomicInteger(0);
        everyKw.forEach(item->{
            int kw = Integer.parseInt(item.get("kw"));
            int zt = Integer.parseInt(item.get("zt"));
            if (zt>=11){
                sl.incrementAndGet();
            }
            if (zt>=42){
                slc.incrementAndGet();
            }
            if (kw==1){
                if (zt>=42){
                    kw1.incrementAndGet();
                }
                if (zt==52||zt==62){
                    kw1c.incrementAndGet();
                }
            }
            if (kw==2){
                if (zt>=42){
                    kw2.incrementAndGet();
                }
                if (zt==52||zt==62){
                    kw2c.incrementAndGet();
                }
            }
            if (kw==3){
                if (zt>=42){
                    kw3.incrementAndGet();
                }
                if (zt==52||zt==62){
                    kw3c.incrementAndGet();
                }
            }
            if (kw==4){
                if (zt>=42){
                    kw4.incrementAndGet();
                }
                if (zt==52||zt==62){
                    kw4c.incrementAndGet();
                }
            }
        });
        List<Integer> zsl = Arrays.asList(sl.get(),kw1.get(),kw2.get(),kw3.get(),kw4.get());
        List<Integer> kwc = Arrays.asList(slc.get(),kw2c.get(),kw3c.get(),kw4c.get());
        List<Integer> wcl = Arrays.asList(sl.get()-slc.get(),kw2.get()-kw2c.get(),kw3.get()-kw3c.get(),kw4.get()-kw4c.get());
        map.put("zsl",zsl);//总数量
        map.put("kwc",kwc);//完成数量
        map.put("wcl",wcl);//未处理
        return map;
    }

    public List<Double> getEveryMonXbzlByNf(String nf) {
        return kbanMapper.getEveryMonXbzlByNf(nf);
    }

    public List<Map<String,Object>> getEveryCompZlByNf(String nf) {
        DateTime from = DateUtil.beginOfYear(DateUtil.parse(nf, "yyyy"));
        DateTime to = DateUtil.endOfYear(DateUtil.parse(nf, "yyyy"));
        return kbanMapper.getEveryCompZlByNf(from.toString("yyyy/MM/dd"), to.toString("yyyy/MM/dd"));
    }

}
